# Copyright 2021 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""transform pytorch checkpoint to mindspore checkpoint"""
import os
import argparse

import torch
from mindspore import Tensor
from mindspore.train.serialization import save_checkpoint

parser = argparse.ArgumentParser(description="transform pytorch checkpoint to mindspore checkpoint")
parser.add_argument("--torch_file", type=str, required=True, help="input pytorch checkpoint filename")
parser.add_argument("--output_path", type=str, required=True, help="output mindspore checkpoint path")
args = parser.parse_args()

weights_dict = {
    "feature_extractor.conv1.weight": "conv1.weight",
    "feature_extractor.bn1.moving_mean": "bn1.running_mean",
    "feature_extractor.bn1.moving_variance": "bn1.running_var",
    "feature_extractor.bn1.gamma": "bn1.weight",
    "feature_extractor.bn1.beta": "bn1.bias",

    "feature_extractor.layer1.0.conv1.weight": "layer1.0.conv1.weight",
    "feature_extractor.layer1.0.bn1.moving_mean": "layer1.0.bn1.running_mean",
    "feature_extractor.layer1.0.bn1.moving_variance": "layer1.0.bn1.running_var",
    "feature_extractor.layer1.0.bn1.gamma": "layer1.0.bn1.weight",
    "feature_extractor.layer1.0.bn1.beta": "layer1.0.bn1.bias",
    "feature_extractor.layer1.0.conv2.weight": "layer1.0.conv2.weight",
    "feature_extractor.layer1.0.bn2.moving_mean": "layer1.0.bn2.running_mean",
    "feature_extractor.layer1.0.bn2.moving_variance": "layer1.0.bn2.running_var",
    "feature_extractor.layer1.0.bn2.gamma": "layer1.0.bn2.weight",
    "feature_extractor.layer1.0.bn2.beta": "layer1.0.bn2.bias",
    "feature_extractor.layer1.0.conv3.weight": "layer1.0.conv3.weight",
    "feature_extractor.layer1.0.bn3.moving_mean": "layer1.0.bn3.running_mean",
    "feature_extractor.layer1.0.bn3.moving_variance": "layer1.0.bn3.running_var",
    "feature_extractor.layer1.0.bn3.gamma": "layer1.0.bn3.weight",
    "feature_extractor.layer1.0.bn3.beta": "layer1.0.bn3.bias",
    "feature_extractor.layer1.0.conv_down_sample.weight": "layer1.0.downsample.0.weight",
    "feature_extractor.layer1.0.bn_down_sample.moving_mean": "layer1.0.downsample.1.running_mean",
    "feature_extractor.layer1.0.bn_down_sample.moving_variance": "layer1.0.downsample.1.running_var",
    "feature_extractor.layer1.0.bn_down_sample.gamma": "layer1.0.downsample.1.weight",
    "feature_extractor.layer1.0.bn_down_sample.beta": "layer1.0.downsample.1.bias",

    "feature_extractor.layer1.1.conv1.weight": "layer1.1.conv1.weight",
    "feature_extractor.layer1.1.bn1.moving_mean": "layer1.1.bn1.running_mean",
    "feature_extractor.layer1.1.bn1.moving_variance": "layer1.1.bn1.running_var",
    "feature_extractor.layer1.1.bn1.gamma": "layer1.1.bn1.weight",
    "feature_extractor.layer1.1.bn1.beta": "layer1.1.bn1.bias",
    "feature_extractor.layer1.1.conv2.weight": "layer1.1.conv2.weight",
    "feature_extractor.layer1.1.bn2.moving_mean": "layer1.1.bn2.running_mean",
    "feature_extractor.layer1.1.bn2.moving_variance": "layer1.1.bn2.running_var",
    "feature_extractor.layer1.1.bn2.gamma": "layer1.1.bn2.weight",
    "feature_extractor.layer1.1.bn2.beta": "layer1.1.bn2.bias",
    "feature_extractor.layer1.1.conv3.weight": "layer1.1.conv3.weight",
    "feature_extractor.layer1.1.bn3.moving_mean": "layer1.1.bn3.running_mean",
    "feature_extractor.layer1.1.bn3.moving_variance": "layer1.1.bn3.running_var",
    "feature_extractor.layer1.1.bn3.gamma": "layer1.1.bn3.weight",
    "feature_extractor.layer1.1.bn3.beta": "layer1.1.bn3.bias",

    "feature_extractor.layer1.2.conv1.weight": "layer1.2.conv1.weight",
    "feature_extractor.layer1.2.bn1.moving_mean": "layer1.2.bn1.running_mean",
    "feature_extractor.layer1.2.bn1.moving_variance": "layer1.2.bn1.running_var",
    "feature_extractor.layer1.2.bn1.gamma": "layer1.2.bn1.weight",
    "feature_extractor.layer1.2.bn1.beta": "layer1.2.bn1.bias",
    "feature_extractor.layer1.2.conv2.weight": "layer1.2.conv2.weight",
    "feature_extractor.layer1.2.bn2.moving_mean": "layer1.2.bn2.running_mean",
    "feature_extractor.layer1.2.bn2.moving_variance": "layer1.2.bn2.running_var",
    "feature_extractor.layer1.2.bn2.gamma": "layer1.2.bn2.weight",
    "feature_extractor.layer1.2.bn2.beta": "layer1.2.bn2.bias",
    "feature_extractor.layer1.2.conv3.weight": "layer1.2.conv3.weight",
    "feature_extractor.layer1.2.bn3.moving_mean": "layer1.2.bn3.running_mean",
    "feature_extractor.layer1.2.bn3.moving_variance": "layer1.2.bn3.running_var",
    "feature_extractor.layer1.2.bn3.gamma": "layer1.2.bn3.weight",
    "feature_extractor.layer1.2.bn3.beta": "layer1.2.bn3.bias",

    "feature_extractor.layer2.0.conv1.weight": "layer2.0.conv1.weight",
    "feature_extractor.layer2.0.bn1.moving_mean": "layer2.0.bn1.running_mean",
    "feature_extractor.layer2.0.bn1.moving_variance": "layer2.0.bn1.running_var",
    "feature_extractor.layer2.0.bn1.gamma": "layer2.0.bn1.weight",
    "feature_extractor.layer2.0.bn1.beta": "layer2.0.bn1.bias",
    "feature_extractor.layer2.0.conv2.weight": "layer2.0.conv2.weight",
    "feature_extractor.layer2.0.bn2.moving_mean": "layer2.0.bn2.running_mean",
    "feature_extractor.layer2.0.bn2.moving_variance": "layer2.0.bn2.running_var",
    "feature_extractor.layer2.0.bn2.gamma": "layer2.0.bn2.weight",
    "feature_extractor.layer2.0.bn2.beta": "layer2.0.bn2.bias",
    "feature_extractor.layer2.0.conv3.weight": "layer2.0.conv3.weight",
    "feature_extractor.layer2.0.bn3.moving_mean": "layer2.0.bn3.running_mean",
    "feature_extractor.layer2.0.bn3.moving_variance": "layer2.0.bn3.running_var",
    "feature_extractor.layer2.0.bn3.gamma": "layer2.0.bn3.weight",
    "feature_extractor.layer2.0.bn3.beta": "layer2.0.bn3.bias",
    "feature_extractor.layer2.0.conv_down_sample.weight": "layer2.0.downsample.0.weight",
    "feature_extractor.layer2.0.bn_down_sample.moving_mean": "layer2.0.downsample.1.running_mean",
    "feature_extractor.layer2.0.bn_down_sample.moving_variance": "layer2.0.downsample.1.running_var",
    "feature_extractor.layer2.0.bn_down_sample.gamma": "layer2.0.downsample.1.weight",
    "feature_extractor.layer2.0.bn_down_sample.beta": "layer2.0.downsample.1.bias",

    "feature_extractor.layer2.1.conv1.weight": "layer2.1.conv1.weight",
    "feature_extractor.layer2.1.bn1.moving_mean": "layer2.1.bn1.running_mean",
    "feature_extractor.layer2.1.bn1.moving_variance": "layer2.1.bn1.running_var",
    "feature_extractor.layer2.1.bn1.gamma": "layer2.1.bn1.weight",
    "feature_extractor.layer2.1.bn1.beta": "layer2.1.bn1.bias",
    "feature_extractor.layer2.1.conv2.weight": "layer2.1.conv2.weight",
    "feature_extractor.layer2.1.bn2.moving_mean": "layer2.1.bn2.running_mean",
    "feature_extractor.layer2.1.bn2.moving_variance": "layer2.1.bn2.running_var",
    "feature_extractor.layer2.1.bn2.gamma": "layer2.1.bn2.weight",
    "feature_extractor.layer2.1.bn2.beta": "layer2.1.bn2.bias",
    "feature_extractor.layer2.1.conv3.weight": "layer2.1.conv3.weight",
    "feature_extractor.layer2.1.bn3.moving_mean": "layer2.1.bn3.running_mean",
    "feature_extractor.layer2.1.bn3.moving_variance": "layer2.1.bn3.running_var",
    "feature_extractor.layer2.1.bn3.gamma": "layer2.1.bn3.weight",
    "feature_extractor.layer2.1.bn3.beta": "layer2.1.bn3.bias",

    "feature_extractor.layer2.2.conv1.weight": "layer2.2.conv1.weight",
    "feature_extractor.layer2.2.bn1.moving_mean": "layer2.2.bn1.running_mean",
    "feature_extractor.layer2.2.bn1.moving_variance": "layer2.2.bn1.running_var",
    "feature_extractor.layer2.2.bn1.gamma": "layer2.2.bn1.weight",
    "feature_extractor.layer2.2.bn1.beta": "layer2.2.bn1.bias",
    "feature_extractor.layer2.2.conv2.weight": "layer2.2.conv2.weight",
    "feature_extractor.layer2.2.bn2.moving_mean": "layer2.2.bn2.running_mean",
    "feature_extractor.layer2.2.bn2.moving_variance": "layer2.2.bn2.running_var",
    "feature_extractor.layer2.2.bn2.gamma": "layer2.2.bn2.weight",
    "feature_extractor.layer2.2.bn2.beta": "layer2.2.bn2.bias",
    "feature_extractor.layer2.2.conv3.weight": "layer2.2.conv3.weight",
    "feature_extractor.layer2.2.bn3.moving_mean": "layer2.2.bn3.running_mean",
    "feature_extractor.layer2.2.bn3.moving_variance": "layer2.2.bn3.running_var",
    "feature_extractor.layer2.2.bn3.gamma": "layer2.2.bn3.weight",
    "feature_extractor.layer2.2.bn3.beta": "layer2.2.bn3.bias",

    "feature_extractor.layer2.3.conv1.weight": "layer2.3.conv1.weight",
    "feature_extractor.layer2.3.bn1.moving_mean": "layer2.3.bn1.running_mean",
    "feature_extractor.layer2.3.bn1.moving_variance": "layer2.3.bn1.running_var",
    "feature_extractor.layer2.3.bn1.gamma": "layer2.3.bn1.weight",
    "feature_extractor.layer2.3.bn1.beta": "layer2.3.bn1.bias",
    "feature_extractor.layer2.3.conv2.weight": "layer2.3.conv2.weight",
    "feature_extractor.layer2.3.bn2.moving_mean": "layer2.3.bn2.running_mean",
    "feature_extractor.layer2.3.bn2.moving_variance": "layer2.3.bn2.running_var",
    "feature_extractor.layer2.3.bn2.gamma": "layer2.3.bn2.weight",
    "feature_extractor.layer2.3.bn2.beta": "layer2.3.bn2.bias",
    "feature_extractor.layer2.3.conv3.weight": "layer2.3.conv3.weight",
    "feature_extractor.layer2.3.bn3.moving_mean": "layer2.3.bn3.running_mean",
    "feature_extractor.layer2.3.bn3.moving_variance": "layer2.3.bn3.running_var",
    "feature_extractor.layer2.3.bn3.gamma": "layer2.3.bn3.weight",
    "feature_extractor.layer2.3.bn3.beta": "layer2.3.bn3.bias",

    "feature_extractor.layer3.0.conv1.weight": "layer3.0.conv1.weight",
    "feature_extractor.layer3.0.bn1.moving_mean": "layer3.0.bn1.running_mean",
    "feature_extractor.layer3.0.bn1.moving_variance": "layer3.0.bn1.running_var",
    "feature_extractor.layer3.0.bn1.gamma": "layer3.0.bn1.weight",
    "feature_extractor.layer3.0.bn1.beta": "layer3.0.bn1.bias",
    "feature_extractor.layer3.0.conv2.weight": "layer3.0.conv2.weight",
    "feature_extractor.layer3.0.bn2.moving_mean": "layer3.0.bn2.running_mean",
    "feature_extractor.layer3.0.bn2.moving_variance": "layer3.0.bn2.running_var",
    "feature_extractor.layer3.0.bn2.gamma": "layer3.0.bn2.weight",
    "feature_extractor.layer3.0.bn2.beta": "layer3.0.bn2.bias",
    "feature_extractor.layer3.0.conv3.weight": "layer3.0.conv3.weight",
    "feature_extractor.layer3.0.bn3.moving_mean": "layer3.0.bn3.running_mean",
    "feature_extractor.layer3.0.bn3.moving_variance": "layer3.0.bn3.running_var",
    "feature_extractor.layer3.0.bn3.gamma": "layer3.0.bn3.weight",
    "feature_extractor.layer3.0.bn3.beta": "layer3.0.bn3.bias",
    "feature_extractor.layer3.0.conv_down_sample.weight": "layer3.0.downsample.0.weight",
    "feature_extractor.layer3.0.bn_down_sample.moving_mean": "layer3.0.downsample.1.running_mean",
    "feature_extractor.layer3.0.bn_down_sample.moving_variance": "layer3.0.downsample.1.running_var",
    "feature_extractor.layer3.0.bn_down_sample.gamma": "layer3.0.downsample.1.weight",
    "feature_extractor.layer3.0.bn_down_sample.beta": "layer3.0.downsample.1.bias",

    "feature_extractor.layer3.1.conv1.weight": "layer3.1.conv1.weight",
    "feature_extractor.layer3.1.bn1.moving_mean": "layer3.1.bn1.running_mean",
    "feature_extractor.layer3.1.bn1.moving_variance": "layer3.1.bn1.running_var",
    "feature_extractor.layer3.1.bn1.gamma": "layer3.1.bn1.weight",
    "feature_extractor.layer3.1.bn1.beta": "layer3.1.bn1.bias",
    "feature_extractor.layer3.1.conv2.weight": "layer3.1.conv2.weight",
    "feature_extractor.layer3.1.bn2.moving_mean": "layer3.1.bn2.running_mean",
    "feature_extractor.layer3.1.bn2.moving_variance": "layer3.1.bn2.running_var",
    "feature_extractor.layer3.1.bn2.gamma": "layer3.1.bn2.weight",
    "feature_extractor.layer3.1.bn2.beta": "layer3.1.bn2.bias",
    "feature_extractor.layer3.1.conv3.weight": "layer3.1.conv3.weight",
    "feature_extractor.layer3.1.bn3.moving_mean": "layer3.1.bn3.running_mean",
    "feature_extractor.layer3.1.bn3.moving_variance": "layer3.1.bn3.running_var",
    "feature_extractor.layer3.1.bn3.gamma": "layer3.1.bn3.weight",
    "feature_extractor.layer3.1.bn3.beta": "layer3.1.bn3.bias",

    "feature_extractor.layer3.2.conv1.weight": "layer3.2.conv1.weight",
    "feature_extractor.layer3.2.bn1.moving_mean": "layer3.2.bn1.running_mean",
    "feature_extractor.layer3.2.bn1.moving_variance": "layer3.2.bn1.running_var",
    "feature_extractor.layer3.2.bn1.gamma": "layer3.2.bn1.weight",
    "feature_extractor.layer3.2.bn1.beta": "layer3.2.bn1.bias",
    "feature_extractor.layer3.2.conv2.weight": "layer3.2.conv2.weight",
    "feature_extractor.layer3.2.bn2.moving_mean": "layer3.2.bn2.running_mean",
    "feature_extractor.layer3.2.bn2.moving_variance": "layer3.2.bn2.running_var",
    "feature_extractor.layer3.2.bn2.gamma": "layer3.2.bn2.weight",
    "feature_extractor.layer3.2.bn2.beta": "layer3.2.bn2.bias",
    "feature_extractor.layer3.2.conv3.weight": "layer3.2.conv3.weight",
    "feature_extractor.layer3.2.bn3.moving_mean": "layer3.2.bn3.running_mean",
    "feature_extractor.layer3.2.bn3.moving_variance": "layer3.2.bn3.running_var",
    "feature_extractor.layer3.2.bn3.gamma": "layer3.2.bn3.weight",
    "feature_extractor.layer3.2.bn3.beta": "layer3.2.bn3.bias",

    "feature_extractor.layer3.3.conv1.weight": "layer3.3.conv1.weight",
    "feature_extractor.layer3.3.bn1.moving_mean": "layer3.3.bn1.running_mean",
    "feature_extractor.layer3.3.bn1.moving_variance": "layer3.3.bn1.running_var",
    "feature_extractor.layer3.3.bn1.gamma": "layer3.3.bn1.weight",
    "feature_extractor.layer3.3.bn1.beta": "layer3.3.bn1.bias",
    "feature_extractor.layer3.3.conv2.weight": "layer3.3.conv2.weight",
    "feature_extractor.layer3.3.bn2.moving_mean": "layer3.3.bn2.running_mean",
    "feature_extractor.layer3.3.bn2.moving_variance": "layer3.3.bn2.running_var",
    "feature_extractor.layer3.3.bn2.gamma": "layer3.3.bn2.weight",
    "feature_extractor.layer3.3.bn2.beta": "layer3.3.bn2.bias",
    "feature_extractor.layer3.3.conv3.weight": "layer3.3.conv3.weight",
    "feature_extractor.layer3.3.bn3.moving_mean": "layer3.3.bn3.running_mean",
    "feature_extractor.layer3.3.bn3.moving_variance": "layer3.3.bn3.running_var",
    "feature_extractor.layer3.3.bn3.gamma": "layer3.3.bn3.weight",
    "feature_extractor.layer3.3.bn3.beta": "layer3.3.bn3.bias",

    "feature_extractor.layer3.4.conv1.weight": "layer3.4.conv1.weight",
    "feature_extractor.layer3.4.bn1.moving_mean": "layer3.4.bn1.running_mean",
    "feature_extractor.layer3.4.bn1.moving_variance": "layer3.4.bn1.running_var",
    "feature_extractor.layer3.4.bn1.gamma": "layer3.4.bn1.weight",
    "feature_extractor.layer3.4.bn1.beta": "layer3.4.bn1.bias",
    "feature_extractor.layer3.4.conv2.weight": "layer3.4.conv2.weight",
    "feature_extractor.layer3.4.bn2.moving_mean": "layer3.4.bn2.running_mean",
    "feature_extractor.layer3.4.bn2.moving_variance": "layer3.4.bn2.running_var",
    "feature_extractor.layer3.4.bn2.gamma": "layer3.4.bn2.weight",
    "feature_extractor.layer3.4.bn2.beta": "layer3.4.bn2.bias",
    "feature_extractor.layer3.4.conv3.weight": "layer3.4.conv3.weight",
    "feature_extractor.layer3.4.bn3.moving_mean": "layer3.4.bn3.running_mean",
    "feature_extractor.layer3.4.bn3.moving_variance": "layer3.4.bn3.running_var",
    "feature_extractor.layer3.4.bn3.gamma": "layer3.4.bn3.weight",
    "feature_extractor.layer3.4.bn3.beta": "layer3.4.bn3.bias",

    "feature_extractor.layer3.5.conv1.weight": "layer3.5.conv1.weight",
    "feature_extractor.layer3.5.bn1.moving_mean": "layer3.5.bn1.running_mean",
    "feature_extractor.layer3.5.bn1.moving_variance": "layer3.5.bn1.running_var",
    "feature_extractor.layer3.5.bn1.gamma": "layer3.5.bn1.weight",
    "feature_extractor.layer3.5.bn1.beta": "layer3.5.bn1.bias",
    "feature_extractor.layer3.5.conv2.weight": "layer3.5.conv2.weight",
    "feature_extractor.layer3.5.bn2.moving_mean": "layer3.5.bn2.running_mean",
    "feature_extractor.layer3.5.bn2.moving_variance": "layer3.5.bn2.running_var",
    "feature_extractor.layer3.5.bn2.gamma": "layer3.5.bn2.weight",
    "feature_extractor.layer3.5.bn2.beta": "layer3.5.bn2.bias",
    "feature_extractor.layer3.5.conv3.weight": "layer3.5.conv3.weight",
    "feature_extractor.layer3.5.bn3.moving_mean": "layer3.5.bn3.running_mean",
    "feature_extractor.layer3.5.bn3.moving_variance": "layer3.5.bn3.running_var",
    "feature_extractor.layer3.5.bn3.gamma": "layer3.5.bn3.weight",
    "feature_extractor.layer3.5.bn3.beta": "layer3.5.bn3.bias",

    "feature_extractor.layer4.0.conv1.weight": "layer4.0.conv1.weight",
    "feature_extractor.layer4.0.bn1.moving_mean": "layer4.0.bn1.running_mean",
    "feature_extractor.layer4.0.bn1.moving_variance": "layer4.0.bn1.running_var",
    "feature_extractor.layer4.0.bn1.gamma": "layer4.0.bn1.weight",
    "feature_extractor.layer4.0.bn1.beta": "layer4.0.bn1.bias",
    "feature_extractor.layer4.0.conv2.weight": "layer4.0.conv2.weight",
    "feature_extractor.layer4.0.bn2.moving_mean": "layer4.0.bn2.running_mean",
    "feature_extractor.layer4.0.bn2.moving_variance": "layer4.0.bn2.running_var",
    "feature_extractor.layer4.0.bn2.gamma": "layer4.0.bn2.weight",
    "feature_extractor.layer4.0.bn2.beta": "layer4.0.bn2.bias",
    "feature_extractor.layer4.0.conv3.weight": "layer4.0.conv3.weight",
    "feature_extractor.layer4.0.bn3.moving_mean": "layer4.0.bn3.running_mean",
    "feature_extractor.layer4.0.bn3.moving_variance": "layer4.0.bn3.running_var",
    "feature_extractor.layer4.0.bn3.gamma": "layer4.0.bn3.weight",
    "feature_extractor.layer4.0.bn3.beta": "layer4.0.bn3.bias",
    "feature_extractor.layer4.0.conv_down_sample.weight": "layer4.0.downsample.0.weight",
    "feature_extractor.layer4.0.bn_down_sample.moving_mean": "layer4.0.downsample.1.running_mean",
    "feature_extractor.layer4.0.bn_down_sample.moving_variance": "layer4.0.downsample.1.running_var",
    "feature_extractor.layer4.0.bn_down_sample.gamma": "layer4.0.downsample.1.weight",
    "feature_extractor.layer4.0.bn_down_sample.beta": "layer4.0.downsample.1.bias",

    "feature_extractor.layer4.1.conv1.weight": "layer4.1.conv1.weight",
    "feature_extractor.layer4.1.bn1.moving_mean": "layer4.1.bn1.running_mean",
    "feature_extractor.layer4.1.bn1.moving_variance": "layer4.1.bn1.running_var",
    "feature_extractor.layer4.1.bn1.gamma": "layer4.1.bn1.weight",
    "feature_extractor.layer4.1.bn1.beta": "layer4.1.bn1.bias",
    "feature_extractor.layer4.1.conv2.weight": "layer4.1.conv2.weight",
    "feature_extractor.layer4.1.bn2.moving_mean": "layer4.1.bn2.running_mean",
    "feature_extractor.layer4.1.bn2.moving_variance": "layer4.1.bn2.running_var",
    "feature_extractor.layer4.1.bn2.gamma": "layer4.1.bn2.weight",
    "feature_extractor.layer4.1.bn2.beta": "layer4.1.bn2.bias",
    "feature_extractor.layer4.1.conv3.weight": "layer4.1.conv3.weight",
    "feature_extractor.layer4.1.bn3.moving_mean": "layer4.1.bn3.running_mean",
    "feature_extractor.layer4.1.bn3.moving_variance": "layer4.1.bn3.running_var",
    "feature_extractor.layer4.1.bn3.gamma": "layer4.1.bn3.weight",
    "feature_extractor.layer4.1.bn3.beta": "layer4.1.bn3.bias",

    "feature_extractor.layer4.2.conv1.weight": "layer4.2.conv1.weight",
    "feature_extractor.layer4.2.bn1.moving_mean": "layer4.2.bn1.running_mean",
    "feature_extractor.layer4.2.bn1.moving_variance": "layer4.2.bn1.running_var",
    "feature_extractor.layer4.2.bn1.gamma": "layer4.2.bn1.weight",
    "feature_extractor.layer4.2.bn1.beta": "layer4.2.bn1.bias",
    "feature_extractor.layer4.2.conv2.weight": "layer4.2.conv2.weight",
    "feature_extractor.layer4.2.bn2.moving_mean": "layer4.2.bn2.running_mean",
    "feature_extractor.layer4.2.bn2.moving_variance": "layer4.2.bn2.running_var",
    "feature_extractor.layer4.2.bn2.gamma": "layer4.2.bn2.weight",
    "feature_extractor.layer4.2.bn2.beta": "layer4.2.bn2.bias",
    "feature_extractor.layer4.2.conv3.weight": "layer4.2.conv3.weight",
    "feature_extractor.layer4.2.bn3.moving_mean": "layer4.2.bn3.running_mean",
    "feature_extractor.layer4.2.bn3.moving_variance": "layer4.2.bn3.running_var",
    "feature_extractor.layer4.2.bn3.gamma": "layer4.2.bn3.weight",
    "feature_extractor.layer4.2.bn3.beta": "layer4.2.bn3.bias",
}

torch_param_dict = torch.load(args.torch_file)
ms_params = []

for ms_name, torch_name in weights_dict.items():
    if torch_name not in torch_param_dict.keys():
        print("param {} not in pytorch checkpoint".format(torch_name))
        continue
    torch_value = torch_param_dict[torch_name]
    np_value = torch_value.data.numpy()
    each_param = dict()
    each_param["name"] = ms_name
    each_param["data"] = Tensor(np_value)
    ms_params.append(each_param)

save_checkpoint(ms_params, os.path.join(args.output_path, "pretrained_model.ckpt"))
