#!/bin/bash
# Copyright 2021 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================

echo "=============================================================================================================="
echo "Please run the script as: "
echo "bash run_standalone_train_gpu.sh PRE_TRAIN_DATASET DEVICE_ID"
echo "for example:"
echo "bash run_standalone_train_gpu.sh \
  /home/workspace/dataset_menu/train.tok.clean.bpe.32000.en.mindrecord 0"
echo "It is better to use absolute path."
echo "=============================================================================================================="

PRE_TRAIN_DATASET=$1

export GLOG_v=2
export CUDA_VISIBLE_DEVICES=$2

current_exec_path=$(pwd)
echo ${current_exec_path}
if [ -d "train" ];
then
    rm -rf ./train
fi
mkdir ./train
cp ../*.py ./train
cp ../*.yaml ./train
cp -r ../src ./train
cp -r ../model_utils ./train
cd ./train || exit
echo "start for training"
env > env.log

config_path="${current_exec_path}/train/default_config_gpu.yaml"
echo "config path is : ${config_path}"

python train.py \
  --config_path=$config_path \
  --pre_train_dataset=$PRE_TRAIN_DATASET \
  --device_id=0  \
  --device_target="GPU"  > log_gnmt_network.log 2>&1 &
cd ..
